package server

import (
	"log"
	"github.com/spf13/viper"
)

func SetConfigValues() {
	// Set Default Values For Config Variables

	// Vereign API Related
	viper.SetDefault("grpcListenAddress", "localhost:7877")
	viper.SetDefault("restListenAddress", "localhost:7878")
	viper.SetDefault("dataStorageUrl", "localhost:7777")
	
	// Certificates Related
	viper.SetDefault("certDir", "cert")
	viper.SetDefault("certFile", "server.crt")
	viper.SetDefault("certKey", "server.key")
	viper.SetDefault("caCertFile", "ca.crt")
	viper.SetDefault("vereignCertFile", "vereign_ca.cer")
	viper.SetDefault("vereignCertKey", "vereign_ca.key")

	viper.SetDefault("viamUUID", "viam-system")
	viper.SetDefault("viamSession", "viam-session")

	viper.SetDefault("maxMessageSize", "32")

	// Read Config File
	viper.SetConfigName("config")
	viper.AddConfigPath(".")
	if err := viper.ReadInConfig(); err != nil {
		log.Printf("can't read config: %s, will use default values", err)
	}

	// Print all config values to log file
	log.Printf("All Settings From Config:")
	as := viper.AllSettings()
	for key, _ := range as {
	    log.Printf("%s => %s", key, viper.GetString(key))
	}
}